#ifndef _CCode_CPP
#define _CCode_CPP
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <StdIO.H>

#include "CCode.H"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool InitializeCryptography(LPSIMPLECIPHER lpSimpleCipher, char *sKeyBuf, int iKeySz)
{
	int iKeyPos = 0;

	//Save the size of the users key.
	lpSimpleCipher->iKeySz = iKeySz;

	//Allocate enough RAM to store the users key.
	lpSimpleCipher->sKeyBuf = (char *) calloc(sizeof(char), lpSimpleCipher->iKeySz + 1);
	
	//Dont just copy the key in sKeyBuf[] to lpSimpleCipher->sKeyBuf[], scramble it up.
	while(iKeyPos < lpSimpleCipher->iKeySz)
	{
		lpSimpleCipher->sKeyBuf[iKeyPos] = (sKeyBuf[iKeyPos] + (sKeyBuf[iKeyPos] ^ iKeyPos));
		iKeyPos++;
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool DestroyCryptography(LPSIMPLECIPHER lpSimpleCipher)
{
	//Free the RAM used by the copy of the users key.
	free(lpSimpleCipher->sKeyBuf);
	lpSimpleCipher->sKeyBuf = NULL;

	//ZERO out the size of the users key.
	lpSimpleCipher->iKeySz = 0;
	return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Encode(LPSIMPLECIPHER lpSimpleCipher, char *sInBuf, char *sOutBuf, int iBufSz)
{
	int iDataPos = 0;
	int iKeyPos = 0;

	//Loop through all the users data, off-setting each one with the users key.
	while(iDataPos < iBufSz)
	{
		//Offset the data.
		sOutBuf[iDataPos] = (sInBuf[iDataPos] + lpSimpleCipher->sKeyBuf[iKeyPos]);

		if(iKeyPos == lpSimpleCipher->iKeySz)
		{
			iKeyPos = -1;
		}

		iKeyPos++;
		iDataPos++;
	}

	sOutBuf[iDataPos] = '\0';

	return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Decode(LPSIMPLECIPHER lpSimpleCipher, char *sInBuf, char *sOutBuf, int iBufSz)
{
	int iDataPos = 0;
	int iKeyPos = 0;

	//Loop through all the users data, off-setting each one with the users key.
	while(iDataPos < iBufSz)
	{
		//Offset the data.
		sOutBuf[iDataPos] = (sInBuf[iDataPos] - lpSimpleCipher->sKeyBuf[iKeyPos]);

		if(iKeyPos == lpSimpleCipher->iKeySz)
		{
			iKeyPos = -1;
		}

		iKeyPos++;
		iDataPos++;
	}

	sOutBuf[iDataPos] = '\0';

	return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
