#ifndef _Common_CPP
#define _Common_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

#include "Debug.H"
#include "NSWFL.H"
#include "Common.H"

#include "../SharedClasses/CMemPool/CMemPool.H"
#include "../SharedSource/EventLog.H"

#ifdef _SQLESERVER
	#include "../SQLEServer/Source/Routines.H"
	#include "../SQLEServer/Source/Entry.H"
#elif _SQLECLIENT
	#include "../SQLEClient/Source/Routines.H"
	#include "../SQLEClient/Source/Entry.H"
#else
	#error "You must define either _SQLESERVER or _SQLECLIENT"
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GetListItemlParam(HWND hWnd, int iItem, int iSubItem)
{
	LV_ITEM MyLVItem;

	memset(&MyLVItem, 0, sizeof(MyLVItem));
	MyLVItem.mask       = LVIF_PARAM;
    MyLVItem.iSubItem   = iSubItem;
	MyLVItem.iItem      = iItem;

	if(ListView_GetItem(hWnd, &MyLVItem))
	{
		return MyLVItem.lParam;
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool OpenLogFile(void)
{
    char sFullPath[MAX_PATH];

    SYSTEMTIME ST;

    GetLocalTime(&ST);

    char DateTemp[64 + 1];

    GetDateFormat(LOCALE_USER_DEFAULT, NULL, &ST, "MM-dd-yy", DateTemp,64);

    sprintf(sFullPath, "%s\\%s.txt", gsSessionLogDir, DateTemp);

    if( (gfhSessionLog = fopen(sFullPath, "ab")) == NULL)
        return false;

    return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CloseLogFile(void)
{
	if(gfhSessionLog)
	{
		fclose(gfhSessionLog);
		return true;
	}
	else return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WriteAnEvent(const char *sMessage, int iEventType)
{
	if(iEventType == EVENT_NONE)
	{
		//No need to record an event.
	}
	else if(iEventType == EVENT_INFO)
	{
		RecordInfo(sMessage);
	}
	else if(iEventType == EVENT_WARN)
	{
		RecordWarn(sMessage);
		gdwWarningCount++;
	}
	else if(iEventType == EVENT_ERROR)
	{
		RecordError(sMessage);
		gdwErrorCount++;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WriteSysLogEx(const char *sMessage, int iEventType)
{
    SYSTEMTIME ST;
    GetLocalTime(&ST);

    char lsDate[64];
    char lsTime[64];
	char lsTemp[MAX_STATUS_TEXT + (64 * 2) + 1];

    GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &ST, NULL, lsDate, 64);
    GetTimeFormat(LOCALE_USER_DEFAULT, 0, &ST, NULL, lsTime, 64);

	sprintf(lsTemp, "(%s %s) [*] %s", lsDate, lsTime, sMessage);
	fprintf(gfhSessionLog, "%s\r\n", lsTemp);
 	
	if(!gbCacheLogFile)
	{
		fflush(gfhSessionLog);
	}
    
	InsertListBoxItemEx(MDI.LogList_hWnd, lsTemp, 100, -1);

	WriteAnEvent(lsTemp, iEventType);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WriteSysLog(const char *sMessage)
{
	WriteSysLogEx(sMessage, EVENT_NONE);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WriteLogEx(int iClientId, const char *sMessage, int iEventType)
{
    SYSTEMTIME ST;
    GetLocalTime(&ST);

    char lsDate[64];
    char lsTime[64];
	char lsTemp[MAX_STATUS_TEXT + (64 * 2) + 1];

    GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &ST, NULL, lsDate, 64);
    GetTimeFormat(LOCALE_USER_DEFAULT, 0, &ST, NULL, lsTime, 64);

	sprintf(lsTemp, "(%s %s) [%d] %s", lsDate, lsTime, iClientId, sMessage);
	fprintf(gfhSessionLog, "%s\r\n", lsTemp);

	if(!gbCacheLogFile)
	{
		fflush(gfhSessionLog);
	}

	InsertListBoxItemEx(MDI.LogList_hWnd, lsTemp, 100, -1);

	WriteAnEvent(lsTemp, iEventType);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WriteLog(int iClientId, const char *sMessage)
{
	WriteLogEx(iClientId, sMessage, EVENT_NONE);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CmdCmp(const char *sBuf, const char *sCmd)
{
	int iCmdLen = strlen(sCmd);
	if( strncmp(sBuf, sCmd, iCmdLen) == 0)
		return iCmdLen;
	else return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int AppendDataToCmd(const char *sCmd, char *sData, int iDataSz, char *sOutBuf)
{
	int iRPos = 0;
	int iRWos = strlen(sCmd);

	strcpy(sOutBuf, sCmd);

	while(iRPos < iDataSz)
		sOutBuf[iRWos++] = sData[iRPos++];

	return iRWos;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int BreakCmdFromData(const char *sBuf, int iCmdLen, int iBufSz, char *sOutBuf)
{
	int WPos = 0;
	int RPos = iCmdLen;

	while(RPos < iBufSz)
		sOutBuf[WPos++] = sBuf[RPos++];

	sOutBuf[WPos] = '\0';

	return WPos;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CreateFolderStructure(char *sFullPath)
{
    char Directory[MAX_PATH];

    int NameLength = strlen(sFullPath);
    int Position = 0;

    while(Position < NameLength)
    {
        if(sFullPath[Position] == '\\' || sFullPath[Position] == '/')
        {
            Directory[Position] = '\0';
            CreateDirectory(Directory, NULL);
        }

        Directory[Position] = sFullPath[Position];

        Position++;
    }
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char *ResolveHost(const char *sHostName, int iAddress)
{
	char *IPAddress = NULL;
	struct hostent * Host;
	struct in_addr Address;
	Host = gethostbyname(sHostName);
	if(Host == NULL)return NULL;
	memcpy(&Address.s_addr,(*Host).h_addr_list[iAddress],(*Host).h_length);
	IPAddress = inet_ntoa(Address);
	return IPAddress;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool GetIPAddress(const char *sInputAddr, char *sOutAddr)
{
	bool ValidIP = true;

    if(inet_addr(sInputAddr) == INADDR_NONE) ValidIP = false;
    if(strlen(sInputAddr) < 7)               ValidIP = false;
    if(strlen(sInputAddr) > 15)              ValidIP = false;

	if(ValidIP == false)
	{
		char *Resolved = ResolveHost(sInputAddr, 0);
		if(Resolved == NULL)
		{
			return false;
		}
		strcpy(sOutAddr, Resolved); 
	}
	else{
		strcpy(sOutAddr, sInputAddr); 
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
